@@ -7,6 +7,8 @@ Bootstrap5Widgets to be used with
 https://getbootstrap.com/docs/5.0/getting-started/introduction/
  
 '''
+from typing import Callable
+
 import justpy as jp
 import os
 import socket
@@ -137,6 +139,12 @@ class App(object):
         errorMsgHtml=f"{errorMsg}<pre>{trace}</pre>"
         self.errors.inner_html=errorMsgHtml
         
+    def clearErrors(self):
+        '''
+        clear the Error area
+        '''
+        self.errors.inner_html=""
+        
     def createSelectorGroupWithLabel(self,a,text:str):
         '''
         create a selector Group with the given label
@@ -172,22 +180,22 @@ class App(object):
         comboBox=ComboBox(a=selectorGroup,**kwargs)
         return comboBox
     
-    def createInput(self,text,placeholder,change,a,size:int=30, **kwargs):
+    def createInput(self,labelText,placeholder,change,a,size:int=30, **kwargs):
         '''
         create an input control with a label with the given text
         a placeholder text and a change onChange function having the parent a
         
         Args:
-            text(str): the text for the label
+            labelText(str): the text for the label
             placeholder(str): a placeholder value
             change(func): an onChange function
             a(object): the parent component of the Select control
             size(int). the size of the input
         '''
-        inputLabel=jp.Label(text=text,a=a,classes="form-label label")
-        jpinput=jp.Input(a=a,classes="form-input",size=size,placeholder=placeholder, **kwargs)
+        selectorGroup,selectorLabel=self.createSelectorGroupWithLabel(a, text=labelText)
+        jpinput=jp.Input(a=selectorGroup,classes="form-input",size=size,placeholder=placeholder, **kwargs)
         jpinput.on('input', change)
-        inputLabel.for_component=inputLabel
+        selectorLabel.for_component=jpinput
         return jpinput
         
     def cmdLine(self,argv,callback):
@@ -285,6 +293,8 @@ class DataList(jp.Div):
         cls = JustpyBaseComponent
         self.id = cls.next_id
         
+    def addOption(self,text,value):
+        self.inner_html+=f"""<option value="{value}">{text}</option>"""
                  
 class ComboBox(jp.Input):
     '''
@@ -307,15 +317,27 @@ class ProgressBar(jp.Div):
     see https://getbootstrap.com/docs/5.0/components/progress/
     """
 
-    def __init__(self, animated:bool=False, **kwargs):
+    def __init__(self, animated:bool=False, withKillSwitch:bool=False, **kwargs):
         '''
         constructor
         '''
-        super().__init__(classes="progress", **kwargs)
+        super().__init__(classes="row align-items-center", **kwargs)
+        if withKillSwitch:
+            self.stopButton:jp.Button = jp.Button(a=self, text="Stop", classes="btn btn-danger col-1")
+        colClass = "col-11" if withKillSwitch else "col-12"
+        self.progressContainer = jp.Div(a=self, classes=colClass)
+        self.progressDiv = jp.Div(a=self.progressContainer, classes="progress")
         self.animated = animated
         self.value = 0
         self.bar = self.getBar()
 
+    def onStop(self, *, callback:Callable):
+        """
+        Callback for the kill switch / stop button
+        """
+        if getattr(self, "stopButton") is not None and isinstance(self.stopButton, jp.Button):
+            self.stopButton.on("click", callback)
+
     def getBar(self):
         """
         generate the actual bar based on the progress
@@ -329,7 +351,7 @@ class ProgressBar(jp.Div):
             "aria-valuenow": self.value,
             "style":f"width: {self.value}%"
         }
-        return jp.Div(a=self, classes=classes, **valueProps)
+        return jp.Div(a=self.progressDiv, classes=classes, **valueProps)
 
     def updateProgress(self, value:int=1):
         """
@@ -337,7 +359,7 @@ class ProgressBar(jp.Div):
         """
         if value >=0 and value <= 100:
             self.value = value
-            self.delete_components()
+            self.progressDiv.delete_components()
             self.bar = self.getBar()
 
     def incrementProgress(self, value: int):
